package com.winhc.phoenix.example.util.company.search;

import org.elasticsearch.index.query.*;
import org.elasticsearch.index.query.functionscore.ScriptScoreFunctionBuilder;
import org.elasticsearch.script.Script;
import org.elasticsearch.script.ScriptType;
import org.elasticsearch.search.rescore.QueryRescoreMode;
import org.elasticsearch.search.rescore.QueryRescorerBuilder;
import org.elasticsearch.search.rescore.RescoreBuilder;

import java.util.*;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * @author: XuJiakai
 * 2021/11/17 17:17
 */
public class CompanySearchQueryUtils {
    public static QueryBuilder getQueryBuilder(CompanyQueryVo companyQueryVo) {

        String content = companyQueryVo.getContent();
        BoolQueryBuilder boolQuery = getBoolQuery(companyQueryVo);

        //以下为过滤逻辑
        BoolQueryBuilder returnBoolQuery = boolQuery()
                .filter(termQuery("deleted", "0"))
                .filter(rangeQuery("company_score_weight").gt(0.3F))
                .must(boolQuery);

        Optional<List<Integer>> searchTypeList = Optional.ofNullable(companyQueryVo.getSearchTypeList());
        if (searchTypeList.isPresent()) {
            BoolQueryBuilder filter = boolQuery();
            searchTypeList.get().forEach(i -> {
                switch (i) {
                    case 1:
                        filter.should(getSpanNearQuery("cname.show.standard", content));
                        filter.should(getSpanNearQuery("history_name.show.standard", content));
                        break;
                    case 2:
                        filter.should(getSpanNearQuery("legal_entity_name.standard", content));
                        break;
                    case 3:
                        filter.should(getSpanNearQuery("holder.name.standard", content));
                        filter.should(getSpanNearQuery("staff.name.standard", content));
                        break;
                    case 4:
                        filter.should(getSpanNearQuery("app_info.standard", content));
                        break;
                    default:
                        break;
                }
            });
            returnBoolQuery.filter(filter);
        }
        return returnBoolQuery;
    }

    public static List<RescoreBuilder> getReScoreBuilder(CompanyQueryVo companyQueryVo) {
        String content = companyQueryVo.getCleanupContent();

        Map<String, Object> map = new HashMap<String, Object>(2) {{
            put("query_content", content);
            put("der", 0.85);
        }};
        List<RescoreBuilder> list = new ArrayList<>();

        //加上名称全匹配分数
        list.add(new QueryRescorerBuilder(functionScoreQuery(new ScriptScoreFunctionBuilder(new Script(ScriptType.STORED, null, "company-name-term-score_v2", map))))
                .windowSize(100)
                .setScoreMode(QueryRescoreMode.Total));

        //乘上权重分
        list.add(new QueryRescorerBuilder(functionScoreQuery(new ScriptScoreFunctionBuilder(new Script(ScriptType.STORED, null, "company-search-script_v2", map))))
                .windowSize(100)
                .setScoreMode(QueryRescoreMode.Multiply));

        //加上知产的少量加分
        list.add(new QueryRescorerBuilder(getRescorerBool(content))
                .windowSize(50)
                .setScoreMode(QueryRescoreMode.Total));

        return list;
    }


    private static BoolQueryBuilder getBoolQuery(CompanyQueryVo companyQueryVo) {
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();
        String org_content = companyQueryVo.getContent();
        String content = CompanyQueryVo.cleanup(companyQueryVo.getContent());

        if (content.length() > 3) {
            boolQuery.should(disMaxQuery()
                    .add(termQuery("history_name.show.keyword", org_content))
                    .add(termQuery("history_name.value", content))
            );


            boolQuery.should(termQuery("org_number", org_content.toUpperCase()).boost(1000));
            boolQuery.should(termQuery("credit_code", org_content.toUpperCase()).boost(1000));
            boolQuery.should(termQuery("reg_number", org_content.toUpperCase()).boost(1000));
            boolQuery.should(termQuery("icp_domain.keyword", org_content).boost(1000));
            boolQuery.should(termQuery("emails.keyword", org_content).boost(1000));
            boolQuery.should(termQuery("phones", org_content).boost(1000));
            boolQuery.should(termQuery("reg_location.keyword", org_content).boost(1000));
        }

        boolQuery.should(disMaxQuery()
                .add(disMaxQuery()
                        .add(termQuery("legal_entity_name.keyword", org_content).boost(10))
                        .add(termQuery("holder.name.keyword", content).boost(10F))
                        .add(termQuery("holder_history.name.keyword", content).boost(10F))
                        .add(termQuery("staff.name.keyword", content).boost(5.5F))
                        .add(termQuery("staff_history.name.keyword", content).boost(5.5F))
                        .tieBreaker(0.3F))
                .add(disMaxQuery()
                        .add(matchQuery("legal_entity_name", org_content).boost(6).minimumShouldMatch("5<95%"))
                        .add(matchPhraseQuery("holder.name", content).boost(10).slop(3))
                        .add(matchPhraseQuery("holder_history.name", content).boost(10).slop(3))
                        .add(matchPhraseQuery("staff.name", content).boost(6).slop(3))
                        .add(matchPhraseQuery("staff_history.name", content).boost(6).slop(3))
                        .tieBreaker(0.3F))
                .tieBreaker(0.3F)
        );
        boolQuery.should(disMaxQuery()
                .add(disMaxQuery()
                        .add(termQuery("icp.keyword", org_content).boost(20))
                        .add(termQuery("app_info.keyword", org_content).boost(40))
                        .add(termQuery("company_tm.keyword", org_content).boost(20))
                        .tieBreaker(0.4F))
                .add(disMaxQuery()
                        .add(matchPhraseQuery("icp", content).boost(8).slop(3))
                        .add(matchPhraseQuery("app_info", content).boost(19).slop(3))
                        .add(matchPhraseQuery("company_tm", org_content).boost(7).slop(1))
                        .add(matchQuery("reg_location", content).boost(1).minimumShouldMatch("100%"))
                        .tieBreaker(0.3F))
                .tieBreaker(0.4F)
        );


        boolQuery.should(disMaxQuery()
                .add(disMaxQuery()
                        .add(matchQuery("emails", org_content).boost(7).minimumShouldMatch("100%"))
                        .add(matchQuery("icp_domain", org_content).boost(1000).minimumShouldMatch("100%"))

                        .tieBreaker(0.3F))
                .tieBreaker(0.4F)
        );


        boolQuery.should(
                disMaxQuery()
                        .add(disMaxQuery()
                                .add(matchPhraseQuery("cname.show.pinyin", content))
                                .add(matchPhraseQuery("history_name.show.pinyin", content))
                        )
                        .add(multiMatchQuery(content)
                                .type(MultiMatchQueryBuilder.Type.CROSS_FIELDS)
                                .minimumShouldMatch("5<90%")
                                .tieBreaker(0.3F)

                                .field("cname.show", 16)
                                .field("history_name.show", 12))
                        .add(multiMatchQuery(content)
                                .operator(Operator.AND)
                                .type(MultiMatchQueryBuilder.Type.CROSS_FIELDS)
                                .tieBreaker(0.3F)
                                .field("cname.show.standard", 16)
                                .field("history_name.show.standard", 12))

                        .tieBreaker(0.4F)
        );

        return boolQuery;
    }


    private static BoolQueryBuilder getRescorerBool(String content) {
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();

        boolQuery.should(disMaxQuery()
                .add(matchQuery("icp", content).boost(8))
                .add(matchQuery("app_info", content).boost(19))
                .add(matchQuery("company_tm", content).boost(20))
                .tieBreaker(0.5F)
        );
        return boolQuery;
    }


    private static QueryBuilder getSpanNearQuery(String fields, String content) {
        SpanNearQueryBuilder spanNearQueryBuilder = spanNearQuery(spanTermQuery(fields, content.charAt(0) + ""), 1);
        for (int i = 1; i < content.length(); i++) {
            spanNearQueryBuilder.addClause(spanTermQuery(fields, content.charAt(i) + ""));
        }
        return spanNearQueryBuilder;
    }

}
